var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/converters/index.ts
var converters_exports = {};
__export(converters_exports, {
  ArrayParam: () => ArrayParam,
  BooleanParam: () => BooleanParam,
  DateParam: () => DateParam,
  NumberParam: () => NumberParam,
  ObjectParam: () => ObjectParam,
  StringParam: () => StringParam
});
module.exports = __toCommonJS(converters_exports);

// src/converters/ArrayParam.ts
function ArrayParam(elementConfig) {
  return {
    serialize: (values) => {
      if (!values) {
        return null;
      }
      return values.map(elementConfig.serialize);
    },
    deserialize: (values) => {
      if (!values) {
        return null;
      }
      if (!Array.isArray(values)) {
        return [elementConfig.deserialize(values)];
      }
      return values.map(elementConfig.deserialize);
    }
  };
}

// src/converters/BooleanParam.ts
var BooleanParam = {
  serialize: (value) => typeof value === "boolean" ? value.toString() : null,
  deserialize: (value) => {
    if (value === "true")
      return true;
    if (value === "false")
      return false;
    return null;
  }
};

// src/converters/DateParam.ts
var DateParam = {
  serialize: (value) => (value == null ? void 0 : value.toISOString()) ?? null,
  deserialize: (value) => {
    const stringValue = value;
    try {
      const date = new Date(stringValue);
      return isNaN(date.getTime()) ? null : date;
    } catch {
      return null;
    }
  }
};

// src/converters/NumberParam.ts
var NumberParam = {
  serialize: (value) => Number.isFinite(value) ? `${value}` : null,
  deserialize: (value) => {
    try {
      const parsedValue = Number.parseFloat(value);
      return Number.isFinite(parsedValue) ? parsedValue : null;
    } catch {
      return null;
    }
  }
};

// src/converters/ObjectParam.ts
function ObjectParam() {
  return {
    serialize: (value) => value === null ? null : encodeURIComponent(JSON.stringify(value)),
    deserialize: (value) => {
      if (value === null || Array.isArray(value)) {
        return null;
      }
      try {
        return JSON.parse(decodeURIComponent(value));
      } catch {
        return null;
      }
    }
  };
}

// src/converters/StringParam.ts
var StringParam = {
  serialize: (value) => value,
  deserialize: (value) => value ?? null
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ArrayParam,
  BooleanParam,
  DateParam,
  NumberParam,
  ObjectParam,
  StringParam
});
//# sourceMappingURL=index.js.map